'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fsPromise = require('fs-promise');

var _fsPromise2 = _interopRequireDefault(_fsPromise);

var _isEqual = require('is-equal');

var _isEqual2 = _interopRequireDefault(_isEqual);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const logger = console;

class AxioxRecorder {
  constructor(options = {}) {
    this.serverFile = options.serverFile || 'server.json';
    this.prefix = options.prefix || '';
    this.enabled = options.enabled || true;
    this.excludes = options.excludes || { excludePostBody: false };
    this.persistHandler = options.persistHandler;
    this.mocks = [];
  }

  /**
   * Records the request/response.
   *
   * @param {Object} response the response object
   *
   * @returns {Object} the response
   */
  record(response) {
    var _this = this;

    return _asyncToGenerator(function* () {
      const method = response.config.method.toLowerCase();
      const path = _this._getPath(response.config.url);

      const mock = {
        request: {
          path: `${_this.prefix}/${path}`,
          headers: _this._filterHeaders(response.config.headers),
          method
        },
        response: {
          code: response.status,
          headers: response.headers,
          body: response.data
        }
      };

      if (method === 'post' || method === 'put') {
        mock.request.body = response.config.data;

        try {
          if (typeof mock.request.body === 'string' && typeof mock.response.body === 'object') {
            mock.request.body = JSON.parse(mock.request.body);
          }
        } catch (error) {
          logger.error('Unable to parse request body');
        }
      }

      _this.mocks.push(mock);

      yield _this._writeMocks(_this.mocks);

      if (typeof _this.persistHandler === 'function') {
        yield _this.persistHandler(_this.mocks);
      }

      return response;
    })();
  }

  /**
   * Registers recorder with Axios instance.
   *
   * @param {Object} axios   axios instance
   * @param {Object} options recording options
   *
   * @returns {Object} the axios instance
   */
  register(axios, options = {}) {
    if (options.prefix) {
      this.prefix = options.prefix;
    }

    if (this.enabled) {
      axios.interceptors.response.use(this.record.bind(this));
      this._getPath.bind(this);
    }

    return axios;
  }

  /**
   * Retrieve the request path from the url.
   *
   * @param {String} url the request url
   *
   * @returns {String} the request path
   *
   * @private
   */
  _getPath(url) {
    const protocolIndex = url.indexOf('//');
    const domainPath = url.substr(protocolIndex + 2);

    return domainPath.substr(domainPath.indexOf('/') + 1);
  }

  /**
   * Extract request headers object.
   *
   * @param {String} header the request header line
   *
   * @returns {Object} the request header
   *
   * @private
   */
  _filterHeaders(header) {
    const headers = {};

    Object.keys(header).forEach(key => {
      headers[key.toLowerCase()] = header[key];
    });

    return headers;
  }

  /**
   * Writes mocks to a file. This is the default behaviour.
   *
   * @param {Object[]} mocks list of mocks to persist
   *
   * @returns {undefined}
   *
   * @private
   */
  _writeMocks(mocks) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      while (_this2.writing) {
        yield Promise.delay(100);
      }

      _this2.writing = true;

      try {
        let server;
        const path = `${process.cwd()}/${_this2.serverFile}`;

        if (_fsPromise2.default.existsSync(path)) {
          server = yield _fsPromise2.default.readJson(path);
        } else {
          server = {
            mocks: []
          };
        }

        mocks.forEach(function (mock) {
          if (!_this2._mockExists(server.mocks, mock)) {
            logger.log('New Mock:', mock.request.path);
            server.mocks.push(mock);
          }
        });

        yield _fsPromise2.default.writeJson(path, server);
        _this2.writing = false;
      } catch (error) {
        logger.error(error);
        _this2.writing = false;
      }
    })();
  }

  /**
   * Checks whether a mock already exists in list.
   *
   * @param {Object[]} mocks list of mocks
   * @param {Object}   mock  mock to check
   *
   * @returns {Boolean} true if it exists, otherwise false
   *
   * @private
   */
  _mockExists(mocks, mock) {
    return mocks.some(existing => (0, _isEqual2.default)(existing.request, mock.request));
  }
}
exports.default = AxioxRecorder;